#ifndef __IO_UTILITIES__
#define __IO_UTILITIES__

/*! \file IOUtilities.h
 *  \brief Header File for Input/Output Utility Routines
 */

#include <string>

#include "VectorUtilities.h"

using namespace std;

// bool SaveDBLMatrix1D(const DBL_VCTR_1D& mat, const string& File);

// bool SaveDBLMatrix2D(const DBL_VCTR_2D& mat, const string& File);

template < typename T > bool SaveVector1D(vector< T >& V, const string& File)
{
	if (V.size() == 0)
	{
		cerr << "Uninitialized mat - SaveDBLMatrix1D" << endl;
		exit(EXIT_FAILURE);
	}
	int L = static_cast<int>(V.size());
	int l;
	ofstream ofs(File.c_str());
	if (!ofs.good())
	{
		cerr << "Cannot open file " << File << " - SaveVector1D" << endl;
		exit(EXIT_FAILURE);
	}
	for (l = 0; l < L; l++)
	{
		ofs << V[l]<< '\t' ;
	}
	ofs.close();

	return true;
}

template < typename T > bool SaveVector2D(vector< vector< T > >& V, const string& File)
{
	if (V.size() == 0)
	{
		cerr << "Uninitialized mat - SaveVector2D" << endl;
		exit(EXIT_FAILURE);
	}
	int L = static_cast<int>(V.size());
	int C = static_cast<int>(V[0].size());
	int l,c;
	ofstream ofs(File.c_str());
	if (!ofs.good())
	{
		cerr << "Cannot open file " << File << " - SaveVector2D" << endl;
		exit(EXIT_FAILURE);
	}
	for (l = 0; l < L; l++)
	{
		for (c = 0; c < C; c++)
		{
			ofs << V[l][c] << '\t' ;
		}
		ofs << endl;
	}
	ofs.close();

	return true;
}

template < typename T > bool LoadVector2D(vector< vector< T > >& V, const string& File)
{
	if (V.size() == 0)
	{
		cerr << "Uninitialized mat - LoadVector2D" << endl;
		exit(EXIT_FAILURE);
	}
	int L = static_cast<int>(V.size());
	int C = static_cast<int>(V[0].size());
	int l,c;
	ifstream ifs(File.c_str());
	if (!ifs.good())
	{
		cerr << "Cannot open file " << File << " - LoadVector2D" << endl;
		exit(EXIT_FAILURE);
	}
	for (l = 0; l < L; l++)
	{
		for (c = 0; c < C; c++)
		{
			ifs >> V[l][c] ;
		}
	}
	ifs.close();

	return true;
}

//! Saving Genotype Data to a STRUCTURE Format File
/*! \param C Coordinate of Individuals
 *  \param X Genotype Data
 *  \param File The Output File Name
 *  \return Result of the Action (Success/Failure)
 *  \author Durand Eric
 */
bool SaveData(const DBL_VCTR_2D& Xtild, const INT_VCTR_3D& X, const string& File);

void SaveBetaToMyFile(const DBL_VCTR_3D& Beta, const string& File);

//! Loading Genotype Data from a STRUCTURE Format File
/*! \param C Coordinate of Individuals
 *  \param X Genotype Data
 *  \param ER Number of Extra Rows
 *  \param EC Number of Extra Columns
 *  \param File The Input File Name
 *  \return Result of the Action (Success/Failure)
 *  \author Durand Eric
 */
bool LoadData(DBL_VCTR_2D& C, DBL_VCTR_2D& Xtild, INT_VCTR_3D& X, int ER, int EC, const string& File, const bool& specialFormat);

//! Saving Configurations for "Continuing Previous Run"
/*! \param z Membership of Individuals (Hard Clustering)
 *  \param p Estimated Allele Frequencies
 *  \param File The Output File Name
 *  \return Result of the Action (Success/Failure)
 *  \author Durand Eric
 */
bool SaveCF(const DBL_VCTR_1D& tau, const DBL_VCTR_1D& sigma,const DBL_VCTR_2D& beta, const DBL_VCTR_2D& Regression, const DBL_VCTR_2D& CAR, const DBL_VCTR_2D& q, const INT_VCTR_3D& zadmix, const DBL_VCTR_3D& p,const string & File);

//! Loading Configurations for "Continuing Previous Run"
/*! \param z Membership of Individuals (Hard Clustering)
 *  \param p Estimated Allele Frequencies
 *  \param File The Input File Name
 *  \return Result of the Action (Success/Failure)
 *  \author Durand Eric
 */
bool LoadCF(DBL_VCTR_1D& tau, DBL_VCTR_1D& sigma, DBL_VCTR_2D& beta, DBL_VCTR_2D& Regression, DBL_VCTR_2D& CAR, DBL_VCTR_2D& q, INT_VCTR_3D& zadmix, DBL_VCTR_3D& p, string & File);

bool SaveML(const int& trendDeg, const double& minDIC,const string& File);
bool LoadML(int& trendDeg, double& minDIC ,const string& File,bool quiet);

#endif//__IO_UTILITIES__

